#include "camera.h"


void Camera::Render() {
	Vector lookat(pos);
	lookat += view;
	glLoadIdentity();
	gluLookAt(pos.x, pos.y, pos.z, lookat.x, lookat.y, lookat.z, up.x, up.y, up.z);
}

void Camera::RotateX(float amount) {
	view = Vector(view*cos(amount*PIdiv180) + up*sin(amount*PIdiv180));
	view.Normalise();
	//right.y = 0.0f;
	up = view.CrossProduct(right)*-1;
	if(abs(view.DotProduct(Vector(0.0f,1.0f,0.0f)))>0.95f) this->RotateX(-amount);	
}

void Camera::RotateVerticalAxis(float amount) {
	view = Vector(view.x*cos(amount*PIdiv180)+view.z*sin(amount*PIdiv180), view.y, view.z*cos(amount*PIdiv180)-view.x*sin(amount*PIdiv180));
	right = Vector(right.x*cos(amount*PIdiv180)+right.z*sin(amount*PIdiv180), right.y, right.z*cos(amount*PIdiv180)-right.x*sin(amount*PIdiv180));
	up = view.CrossProduct(right)*-1; 
}

void Camera::RotateY(float amount) {
	//rot.y += amount;
	view = Vector(view*cos(amount*PIdiv180) - right*sin(amount*PIdiv180));
	view.Normalise();
	
	right = view.CrossProduct(up);
	right.y = 0.0f;
	right.Normalise();
	up = view.CrossProduct(right)*-1;
}

void Camera::RotateZ(float amount) {
	//rot.z += amount;

	right = Vector(right*cos(amount*PIdiv180) + up*sin(amount*PIdiv180));
	right.y = 0.0f;
	right.Normalise();

	up = view.CrossProduct(right)*-1;
}


void Camera::MoveFwd(float amount) {
	//float y = pos.y;
	pos += view*amount;
	//pos.y = y;
}

void Camera::MoveBack(float amount) {
	//float y = pos.y;
	pos -= view*(amount);
	//pos.y = y;
}

void Camera::MoveLeft(float amount) {
	//float y = pos.y;
	pos -= right*(amount);
	//pos.y = y;
}

void Camera::MoveRight(float amount) {
	//float y = pos.y;
	pos += right*(amount);
	//pos.y = y;
}

void Camera::MoveUp(float amount) {
	pos += up*(amount);
}

void Camera::SetCam(Vector pos, Vector view, Vector up, Vector right) {
	view.Normalise();
	right.Normalise();
	up.Normalise();
	this->pos = pos;
	this->view = view;
	this->right = right;
	this->up = up;
}

void Camera::MoveDown(float amount) {
	pos -= up*(amount);
}

Camera::Camera() {
	pos.Reset();
	//rot.Reset();
	view.Reset();
	view.z = -1.0f;
	right.Reset();
	right.x = 1.0f;
	up.y = 1.0f;
	//camerachanged = false;
}