#include "collisionobject.h"

boolean CollisionBubble::CheckCollision(Vector P, float range) {
	Vector temp = Vector(P.x-M.x,P.y-M.y,P.z-M.z);
	if (temp.GetLength() < r+range) return true;
	else return false;
}

void CollisionBubble::Translate(float x, float y, float z) {
	M+=Vector(x,y,z);
}

boolean CollisionCuboid::CheckCollision(Vector P, float range) {
	boolean inside = true;

	for (int i=2; i<5 && inside; i++) {
		Vector temp = (P-CubeData[0]+CubeData[i]*range);
		temp.Normalise();
		if (temp.DotProduct(CubeData[i]) <= 0) inside = false;
	}

	for (int i=2; i<5 && inside; i++){
		Vector temp = (CubeData[1]-P+CubeData[i]*range);
		temp.Normalise();
		if (temp.DotProduct(CubeData[i]) <= 0) inside = false;
	}

	return inside;
}

void CollisionCuboid::Translate(float x, float y, float z) {
	CubeData[0]+=Vector(x,y,z);
	CubeData[1]+=Vector(x,y,z);
	//printf("%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n%f\n", CubeData[0].x, CubeData[0].y, CubeData[0].z, CubeData[1].x, CubeData[1].y, CubeData[1].z, CubeData[2].x, CubeData[2].y, CubeData[2].z, CubeData[3].x, CubeData[3].y, CubeData[3].z, CubeData[4].x, CubeData[4].y, CubeData[4].z);
}

void CollisionCuboid::RotateX(float amount, Vector center) {
	CubeData[0]-=center;
	CubeData[1]-=center;
	for (int i = 0 ; i < 5 ; i++)
		CubeData[i] = Vector(CubeData[i].x, CubeData[i].y * cos(amount*(float)PIdiv180) + CubeData[i].z * -sin(amount*(float)PIdiv180), CubeData[i].y * sin(amount*(float)PIdiv180) + CubeData[i].z * cos(amount*(float)PIdiv180));
	CubeData[0]+=center;
	CubeData[1]+=center;
}
void CollisionCuboid::RotateY(float amount, Vector center) {
	CubeData[0]-=center;
	CubeData[1]-=center;
	for (int i = 0 ; i < 5 ; i++)
		CubeData[i] = Vector(CubeData[i].x * cos(amount*(float)PIdiv180) + CubeData[i].z * sin(amount*(float)PIdiv180), CubeData[i].y, CubeData[i].x * -sin(amount*(float)PIdiv180) + CubeData[i].z * cos(amount*(float)PIdiv180));
	CubeData[0]+=center;
	CubeData[1]+=center;
}
void CollisionCuboid::RotateZ(float amount, Vector center) {
	CubeData[0]-=center;
	CubeData[1]-=center;
	for (int i = 0 ; i < 5 ; i++)
		CubeData[i] = Vector(CubeData[i].x * cos(amount*(float)PIdiv180) + CubeData[i].y * -sin(amount*(float)PIdiv180), CubeData[i].x * sin(amount*(float)PIdiv180) + CubeData[i].y * cos(amount*(float)PIdiv180), CubeData[i].z);
	CubeData[0]+=center;
	CubeData[1]+=center;
}

int CollisionCollection::insert(CollisionBubble b) {
	bubbles[bubblecount] = b;	
	return bubblecount++;
}
int CollisionCollection::insert(CollisionCuboid c) {
	cubes[cubecount] = c;	
	return cubecount++;
}

CollisionCuboid * CollisionCollection::GetCube(int i) {
	return &cubes[i];
}

CollisionBubble * CollisionCollection::GetBubble(int i) {
	return &bubbles[i];
}

boolean CollisionCollection::CheckCollision(Vector P, float range) {
	boolean collision = false;
	for (int i=0; i<bubblecount && !collision; i++) {
		collision = bubbles[i].CheckCollision(P, range);
	}
	for (int i=0; i<cubecount && !collision; i++) {
		collision = cubes[i].CheckCollision(P, range);
	}
	return collision;
}