#include "model.h"

// magic number "IDP2" or 844121161
#define MD2_IDENT                (('2'<<24) + ('P'<<16) + ('D'<<8) + 'I')

// model version
#define MD2_VERSION              8

// precalculated normal vectors
vec3_t   Model::anorms[ 162 ] = {
#include "anorms.h"
};

float g_angle = 0.0;

GLuint Model::skincount = 0;


float Texture1D[ 16 ] = {0.3f, 0.3f, 0.3f,
						 0.6f, 0.6f, 0.6f, 0.6f, 0.6f,
						 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f};

md2_anim_t Model::animlist[ 21 ] = 
{
    // first, last, fps

	{  2, 10,  9}, // STAND			{   0,  39,  9 },   // STAND 0 39 9
    { 11, 30, 40}, // RUN			{  40,  45, 10 },   // RUN   40 45 10
    { 31, 47, 40}, // ATTACK		{  46,  53, 10 },   // ATTACK 46
    { 48, 57, 20}, // BEHIT			{  54,  57,  7 },   // PAIN_A
    { 57, 70, 40}, // DIE			{  58,  61,  7 },   // PAIN_B
    { 71, 79, 40}, // WALKSIDE		{  62,  65,  7 },   // PAIN_C
    { 80, 90, 40}, // JUMP			{  66,  71,  7 },   // JUMP
    {  72,  83,  7 },   // FLIP
    {  84,  94,  7 },   // SALUTE
    {  95, 111, 10 },   // FALLBACK
    { 112, 122,  7 },   // WAVE
    { 123, 134,  6 },   // POINT
    { 135, 153, 10 },   // CROUCH_STAND
    { 154, 159,  7 },   // CROUCH_WALK
    { 160, 168, 10 },   // CROUCH_ATTACK
    { 196, 172,  7 },   // CROUCH_PAIN
    { 173, 177,  5 },   // CROUCH_DEATH
    { 178, 183,  7 },   // DEATH_FALLBACK
    { 184, 189,  7 },   // DEATH_FALLFORWARD
    { 190, 197,  7 },   // DEATH_FALLBACKSLOW
    { 198, 198,  5 },   // BOOM
};

float    Model::anorms_dots[ 16 ][256] = {
#include    "anormtab.h"
};

bool Model::LoadMD2 (const char *filename) {
{
	std::ifstream   file;           // file stream
	md2_header_t    header;         // md2 header
    char            *buffer;        // buffer storing frame data
    md2_frame_t     *frame;         // temporary variable
    vec3_t          *ptrverts;      // pointer on m_vertices
    int             *ptrnormals;    // pointer on m_lightnormals


    // try to open filename
    file.open( filename, std::ios::in | std::ios::binary );

    if( file.fail() )
        return false;

    // read header file
    file.read( (char *)&header, sizeof( md2_header_t ) );


    /////////////////////////////////////////////
    //      verify that this is a MD2 file

    // check for the ident and the version number

    if( (header.ident != MD2_IDENT) && (header.version != MD2_VERSION) )
    {
        // this is not a MD2 model
        file.close();
        return false;
    }

	/////////////////////////////////////////////

	// initialize member variables
    num_frames  = header.num_frames;
    num_vertices     = header.num_vertices;
    num_glcmds  = header.num_glcmds;


    // allocate memory
    vertices      = new vec3_t[ num_vertices * num_frames ];
    glcmds        = new int[ num_glcmds ];
    lightnormals  = new int[ num_vertices * num_frames ];
    buffer        = new char[ num_frames * header.framesize ];


    /////////////////////////////////////////////
    //          reading file data

    // read frame data...
    file.seekg( header.offset_frames, std::ios::beg );
    file.read( (char *)buffer, num_frames * header.framesize );

    // read opengl commands...
    file.seekg( header.offset_glcmds, std::ios::beg );
    file.read( (char *)glcmds, num_glcmds * sizeof( int ) );

    /////////////////////////////////////////////

	 // vertex array initialization
    for( int j = 0; j < num_frames; j++ )
    {
        // adjust pointers
        frame       = (md2_frame_t *)&buffer[ header.framesize * j ];
		
        ptrverts    = &vertices[ num_vertices * j ];
        ptrnormals  = &lightnormals[ num_vertices * j ];

        for( int i = 0; i < num_vertices; i++ )
        {
            ptrverts[i][0] = (frame->verts[i].v[0] * frame->scale[0]) + frame->translate[0];
            ptrverts[i][1] = (frame->verts[i].v[1] * frame->scale[1]) + frame->translate[1];
            ptrverts[i][2] = (frame->verts[i].v[2] * frame->scale[2]) + frame->translate[2];

            ptrnormals[i] = frame->verts[i].normalIndex;
        }
    }

	// free buffer's memory
    delete [] buffer;

    // close the file and return
    file.close();

	//printf("Load: %ld\n", num_vertices);
	//printf("glcmds: %ld\n", glcmds[0]);

    return true;
}
}


int Model::LoadTex(char* fname) {
	HDC			hdcTemp;												// The DC To Hold Our Bitmap
	HBITMAP		hbmpTemp;												// Holds The Bitmap Temporarily
	IPicture	*pPicture;												// IPicture Interface
	OLECHAR		wszPath[MAX_PATH+1];									// Full Path To Picture (WCHAR)
	char		szPath[MAX_PATH+1];										// Full Path To Picture
	long		lWidth;													// Width In Logical Units
	long		lHeight;												// Height In Logical Units
	GLint		glMaxTexDim ;											// Holds Maximum Texture Size
	BYTE* textureData;
	int width;
	int height;
	bool alpha;
	int colsize;

	if (strlen(fname) >= MAX_IMAGEFILENAME_LENGTH) return -1;
	int i = GetTextureIndex(fname);
	if (i != -1) return i;

	GetCurrentDirectory(MAX_PATH, szPath);						// Get Our Working Directory
	strcat(szPath, "\\");										// Append "\" After The Working Directory
	strcat(szPath, fname);										// Append The PathName

	MultiByteToWideChar(CP_ACP, 0, szPath, -1, wszPath, MAX_PATH);		// Convert From ASCII To Unicode
	HRESULT hr = OleLoadPicturePath(wszPath, 0, 0, 0, IID_IPicture, (void**)&pPicture);

	if(FAILED(hr))														// If Loading Failed
		return -1;													// Return False

	hdcTemp = CreateCompatibleDC(GetDC(0));								// Create The Windows Compatible Device Context
	if(!hdcTemp)														// Did Creation Fail?
	{
		pPicture->Release();											// Decrements IPicture Reference Count
		return -1;													// Return False (Failure)
	}

	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &glMaxTexDim);					// Get Maximum Texture Size Supported
	
	pPicture->get_Width(&lWidth);										// Get IPicture Width (Convert To Pixels)
	width=MulDiv(lWidth, GetDeviceCaps(hdcTemp, LOGPIXELSX), 2540);
	pPicture->get_Height(&lHeight);										// Get IPicture Height (Convert To Pixels)
	height=MulDiv(lHeight, GetDeviceCaps(hdcTemp, LOGPIXELSY), 2540);

	// Resize Image To Closest Power Of Two
	if (width <= glMaxTexDim) // Is Image Width Less Than Or Equal To Cards Limit
		width = 1 << (int)floor((log((double)width)/log(2.0f)) + 0.5f); 
	else  // Otherwise  Set Width To "Max Power Of Two" That The Card Can Handle
		width = 1 << (int)floor((log((double)glMaxTexDim)/log(2.0f)) + 0.5f);
 
	if (height <= glMaxTexDim) // Is Image Height Greater Than Cards Limit
		height = 1 << (int)floor((log((double)height)/log(2.0f)) + 0.5f);
	else  // Otherwise  Set Height To "Max Power Of Two" That The Card Can Handle
		height = 1 << (int)floor((log((double)glMaxTexDim)/log(2.0f)) + 0.5f);
	
	//	Create A Temporary Bitmap
	BITMAPINFO	bi = {0};												// The Type Of Bitmap We Request
	DWORD		*pBits = 0;												// Pointer To The Bitmap Bits

	bi.bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);				// Set Structure Size
	bi.bmiHeader.biBitCount		= 32;									// 32 Bit
	bi.bmiHeader.biWidth		= width;								// Power Of Two Width
	bi.bmiHeader.biHeight		= height;								// Make Image Top Up (Positive Y-Axis)
	bi.bmiHeader.biCompression	= BI_RGB;								// RGB Encoding
	bi.bmiHeader.biPlanes		= 1;									// 1 Bitplane

	//	Creating A Bitmap This Way Allows Us To Specify Color Depth And Gives Us Imediate Access To The Bits
	hbmpTemp = CreateDIBSection(hdcTemp, &bi, DIB_RGB_COLORS, (void**)&pBits, 0, 0);
	
	if(!hbmpTemp)														// Did Creation Fail?
	{
		DeleteDC(hdcTemp);												// Delete The Device Context
		pPicture->Release();											// Decrements IPicture Reference Count
		return -1;													// Return False (Failure)
	}

	SelectObject(hdcTemp, hbmpTemp);									// Select Handle To Our Temp DC And Our Temp Bitmap Object

	// Render The IPicture On To The Bitmap
	pPicture->Render(hdcTemp, 0, 0, width, height, 0, lHeight, lWidth, -lHeight, 0);

	// set the alpha
	alpha=false;
	// and the bit count
	colsize=3;

	ti[textureCount] = (TexInfo*) malloc(sizeof(TexInfo));
	// allocate image
	try {
		textureData = new BYTE [width*height*colsize];
	}
	// did that work?
	catch(std :: bad_alloc)	{
		DeleteObject(hbmpTemp);					// Delete The Object
		DeleteDC(hdcTemp);						// Delete The Device Context
		pPicture->Release();					// Decrements IPicture Reference Count

		colsize=width=height=0;		// clear everything
		return -1;
	}

	// our copy pointer
	BYTE *ptr=textureData;

	// Convert From BGR To RGB Format And Copy over
	for(long i = 0; i < width * height; i++)				// Loop Through All Of The Pixels
	{
		BYTE* pPixel=(BYTE*)(&pBits[i]);					// Grab The Current Pixel
		*(ptr++)=pPixel[2];									// copy red
		*(ptr++)=pPixel[1];									// copy green
		*(ptr++)=pPixel[0];									// copy blue
	}

	DeleteObject(hbmpTemp);									// Delete The Object
	DeleteDC(hdcTemp);										// Delete The Device Context

	pPicture->Release();									// Decrements IPicture Reference Count

	strcpy(ti[textureCount]->name, fname);
	ti[textureCount]->width = width;
	ti[textureCount]->height = height;
	ti[textureCount]->alpha = alpha;
	ti[textureCount]->colsize = colsize;
	
	glGenTextures(1, &(ti[textureCount]->texID));	
	//printf("GenTexture id: %d\n", ti[textureCount]->texID);
	glBindTexture(GL_TEXTURE_2D, ti[textureCount]->texID);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	//glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, ti[textureCount]->width, ti[textureCount]->height, 0, GL_RGB, GL_UNSIGNED_BYTE, textureData);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, ti[textureCount]->width, ti[textureCount]->height,
			GL_RGB, GL_UNSIGNED_BYTE, textureData);


	free(textureData);

	textureCount++;
	texid = ti[textureCount-1]->texID;
	return 1;
}

int Model::GetTextureIndex(char *filename) {
	for (int i=0; i<textureCount; i++)
		if (!strcmp(ti[i]->name, filename)) return i;
	return -1;
}


void Model::MakeFrustumSphere() {
	vec3_t median = {0,0,0};
	float radius;
	vec3_t* vertices = this->GetVertices();
	int num_vertices = this->GetNum_Vertices();
	int num_frames = this->GetNum_Frames();
	float scale = this->GetScale();

	int	frame = 0;

	for (int i = 0; i < num_vertices; i++) {
		median[0] += vertices[i + frame * num_vertices][0];
		median[1] += vertices[i + frame * num_vertices][1];
		median[2] += vertices[i + frame * num_vertices][2];
	}

	median[0] /= num_vertices;
	median[1] /= num_vertices;
	median[2] /= num_vertices;
	
	radius = -100000;
	for (int i = 0; i < num_vertices; i++) {
		float tempr = sqrt(pow(median[0] - vertices[i + frame * num_vertices][0], 2) +
		pow(median[1] - vertices[i + frame * num_vertices][1], 2) +
		pow(median[2] - vertices[i + frame * num_vertices][2], 2));
		if (tempr > radius) radius = tempr;
	}

	frustumCenter = Vector( median[1] * scale, median[2] * scale, median[0] * scale );
	frustumRadius = (float) radius * scale;
}

bool wiref;

void Model::DrawMD2(float time, bool wire, bool light) {
	wiref = wire;
    glPushMatrix();

	    // animate. calculate current frame and next frame
		if( time > 0.0 )
			Animate( time );

		glTranslatef(pos.x, pos.y, pos.z);
		glRotatef(angle.z, 0.0f, 0.0f, 1.0f);
		
		glRotatef(angle.y, 0.0f, 1.0f, 0.0f);
		glRotatef(angle.x, 1.0f, 0.0f, 0.0f);

		// rotate model in the right direction
		//glScalef(0.2f,0.2f,0.2f);

        // rotate the model
        glRotatef( -90.0, 1.0, 0.0, 0.0 );
        glRotatef( -90.0, 0.0, 0.0, 1.0 );

        // render it on the screen
        RenderFrame(light);
    glPopMatrix();
}

void Model::DrawFrame( int frame , bool wire, bool light)
{
    // set new animation parameters...
    anim.startframe   = frame;
    anim.endframe     = frame;
    anim.next_frame   = frame;
    anim.fps          = 1;
    anim.type         = -1;

    // draw the model
    DrawMD2( 1.0, wire, light);
}

void Model::Interpolate( vec3_t *vertlist )
{
    vec3_t  *curr_v;    // pointeur to current frame vertices
    vec3_t  *next_v;    // pointeur to next frame vertices

    // create current frame and next frame's vertex list
    // from the whole vertex list
	
    curr_v = &vertices[ num_vertices * anim.curr_frame ];
    next_v = &vertices[ num_vertices * anim.next_frame ];

    // interpolate and scale vertices to avoid ugly animation
    for( int i = 0; i < num_vertices ; i++ )
    {
        vertlist[i][0] = (curr_v[i][0] + anim.interpol * (next_v[i][0] - curr_v[i][0])) * scale;
		vertlist[i][1] = (curr_v[i][1] + anim.interpol * (next_v[i][1] - curr_v[i][1])) * scale;
        vertlist[i][2] = (curr_v[i][2] + anim.interpol * (next_v[i][2] - curr_v[i][2])) * scale;
    }
}

void Model::RenderFrame(bool light)
{
    static vec3_t    vertlist[16384];  // interpolated vertices
    int              *ptricmds = glcmds;       // pointer on gl commands

	Vector lightDirection = Vector(-1.0f, 0.6f, 0.8f);
	lightDirection.Normalise();
	

    // reverse the orientation of front-facing
    // polygons because gl command list's triangles
    // have clockwise winding
    glPushAttrib( GL_POLYGON_BIT );

	if (wiref)
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glLineWidth(1);
	
	glFrontFace( GL_CW );

    // enable backface culling
    glEnable( GL_CULL_FACE );
    glCullFace( GL_BACK );

    // interpolate
    Interpolate( vertlist );

    // bind model's texture
    glBindTexture( GL_TEXTURE_2D, texid );

    // draw each triangle!
    while( int i = *(ptricmds++) )
    {
        if( i < 0 )
        {
            glBegin( GL_TRIANGLE_FAN );
            i = -i;
        }
        else
        {
            glBegin( GL_TRIANGLE_STRIP );
        }
        for( ; i > 0; i--, ptricmds += 3 )
        {
            // ptricmds[0] : texture coordinate s
            // ptricmds[1] : texture coordinate t
            // ptricmds[2] : vertex index to render

            // parse texture coordinates
            glTexCoord2f( ((float *)ptricmds)[0], ((float *)ptricmds)[1] );
			//vec3_t norm = {anorms[ lightnormals[ ptricmds[2] ]][0], anorms[ lightnormals[ ptricmds[2] ]][1], anorms[ lightnormals[ ptricmds[2] ]][2]};
			if (light) {
				Vector vert = Vector(anorms[ lightnormals[ ptricmds[2] ]][0], anorms[ lightnormals[ ptricmds[2] ]][1], anorms[ lightnormals[ ptricmds[2] ]][2]);
				//vert = Vector(vert.x, vert.y * cos(-angle.x*PIdiv180) + vert.z * -sin(-angle.x*PIdiv180), vert.y * sin(-angle.x*PIdiv180) + vert.z * cos(-angle.x*PIdiv180));
				
				//Ein Model NIE KIPPEN!!! ZERBRECHLICH!!!
				
				//vert = Vector(vert.x * cos(-angle.y*PIdiv180) + vert.z * sin(-angle.y*PIdiv180), vert.y, vert.x * -sin(-angle.y*PIdiv180) + vert.z * cos(-angle.y*PIdiv180));
				vert = Vector(vert.x * cos(angle.y*PIdiv180) + vert.y * -sin(angle.y*PIdiv180), vert.x * sin(angle.y*PIdiv180) + vert.y * cos(angle.y*PIdiv180), vert.z);
				float dot = vert.DotProduct(lightDirection);
				float shadingGrade = Texture1D[(int) ((dot<0)?0:dot * 15.0f)];
				
				//printf("%f\n", shadingGrade);

				glColor3f(shadingGrade, shadingGrade, shadingGrade);
			}
			//glColor3f(anorms_dots[((int)(g_angle * (16 / 360.0))) & (16 - 1)][ lightnormals[ ptricmds[2] ]], anorms_dots[((int)(g_angle * (16 / 360.0))) & (16 - 1)][ lightnormals[ ptricmds[2] ]],anorms_dots[((int)(g_angle * (16 / 360.0))) & (16 - 1)][ lightnormals[ ptricmds[2] ]]);

            // parse triangle's normal (for the lighting)
            //glNormal3fv( anorms[ lightnormals[ ptricmds[2] ]] );

            // draw the vertex
            glVertex3fv( vertlist[ ptricmds[2] ] );

			//vec3_t v = {vertlist[ ptricmds[2] ][0],vertlist[ ptricmds[2] ][1],vertlist[ ptricmds[2] ][2]};
			//vec3_t b = {anorms[ lightnormals[ ptricmds[2] ]][0],anorms[ lightnormals[ ptricmds[2] ]][1],anorms[ lightnormals[ ptricmds[2] ]][2]};
			//vec3_t c = {v[0]+b[0],v[1]+b[1],v[2]+b[2]};
			//glColor3f(0.0,1.0,0.0);
			//glVertex3fv( v );
			//glVertex3fv( c );
			//glColor3f(1.0,1.0,1.0);

        }

        glEnd();
    }

    glDisable( GL_CULL_FACE );


	/////////////////////////////////
	/// Draw Normals

	//glLineWidth(1);

	//ptricmds = glcmds;
 //   // draw each triangle!
 //   while( int i = *(ptricmds++) )
 //   {
 //       if( i < 0 )
 //       {
 //           i = -i;
 //       }
 //       glBegin( GL_LINES );

 //       for( ; i > 0; i--, ptricmds += 3 )
 //       {
	//		vec3_t v = {vertlist[ ptricmds[2] ][0],vertlist[ ptricmds[2] ][1],vertlist[ ptricmds[2] ][2]};
	//		vec3_t b = {anorms[ lightnormals[ ptricmds[2] ]][0],anorms[ lightnormals[ ptricmds[2] ]][1],anorms[ lightnormals[ ptricmds[2] ]][2]};
	//		vec3_t c = {v[0]+b[0],v[1]+b[1],v[2]+b[2]};
	//		glColor3f(0.0,1.0,0.0);
	//		glVertex3fv( v );
	//		glVertex3fv( c );
	//		glColor3f(1.0,1.0,1.0);

 //       }

 //       glEnd();
	//}

	if (!wiref) {

		////////////////////////////////////////////////////
		//Draw Outlines
		//

		ptricmds = glcmds;

		// enable backface culling
		glEnable( GL_CULL_FACE );
		glCullFace( GL_FRONT );

		// bind model's texture
		glBindTexture( GL_TEXTURE_2D, -1 );

		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

		glLineWidth(5);

		glColor3f(0.0,0.0,0.0);

		// draw each triangle!
		while( int i = *(ptricmds++) )
		{
			if( i < 0 )
			{
				glBegin( GL_TRIANGLE_FAN );
				i = -i;
			}
			else
			{
				glBegin( GL_TRIANGLE_STRIP );
			}

			for( ; i > 0; i--, ptricmds += 3 )
			{
				// ptricmds[0] : texture coordinate s
				// ptricmds[1] : texture coordinate t
				// ptricmds[2] : vertex index to render

				// parse texture coordinates

				// parse triangle's normal (for the lighting)
				//glNormal3fv( anorms[ lightnormals[ ptricmds[2] ] ] );

				// draw the vertex
				glVertex3fv( vertlist[ ptricmds[2] ] );
			}

			glEnd();
		}

		glColor3f(1.0,1.0,1.0);

		glDisable( GL_CULL_FACE );
	}
	
	glColor3f(1.0,1.0,1.0);

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

    glPopAttrib();
}

void Model::DrawCoord(){
	glPushMatrix();
	glLineWidth(5.0f);
	glBegin(GL_LINES);
		glColor3f(1.0f,0.0f,0.0f);
		glVertex3f(pos.x, pos.y, pos.z);
		glVertex3f(pos.x+right.x*20, pos.y+right.y*20, pos.z+right.z*20);
		glColor3f(0.0f,0.0f,1.0f);
		glVertex3f(pos.x, pos.y, pos.z);
		glVertex3f(pos.x+up.x*20, pos.y+up.y*20, pos.z+up.z*20);
		glColor3f(0.0f,1.0f,0.0f);
		glVertex3f(pos.x, pos.y, pos.z);
		glVertex3f(pos.x+view.x*20, pos.y+view.y*20, pos.z+view.z*20);
	glEnd();
	glColor3f(1.0f,1.0f,1.0f);
	glPopMatrix();
}

void Model::SetAnim( int type )
{
    if( (type < 0) || (type > MAX_ANIMATIONS) )
        type = 0;

    anim.startframe   = animlist[ type ].first_frame;
    anim.endframe     = animlist[ type ].last_frame;
    anim.next_frame   = animlist[ type ].first_frame + 1;
    anim.fps          = animlist[ type ].fps;
    anim.type         = type;
}

void Model::Animate( float time )
{
    anim.curr_time = time;
	
    // calculate current and next frames
    if( anim.curr_time - anim.old_time > (1.0 / anim.fps) )
    {
		anim.curr_frame = anim.next_frame;
        anim.next_frame++;

        if( anim.next_frame > anim.endframe )
            anim.next_frame = anim.startframe;

        anim.old_time = anim.curr_time;
    }

    // prevent having a current/next frame greater
    // than the total number of frames...
    if( anim.curr_frame > (num_frames - 1) )
        anim.curr_frame = 0;

    if( anim.next_frame > (num_frames - 1) )
        anim.next_frame = 0;

	anim.interpol = anim.fps * (anim.curr_time - anim.old_time);
}

Model::Model() {
	vertices      = 0;
    glcmds        = 0;
    lightnormals  = 0;

    num_frames      = 0;
    num_vertices    = 0;
    num_glcmds      = 0;

    texid         = 0;
    scale         = 1.0;

    SetAnim( 0 );
	//glcmds = NULL;
	//vertices = NULL;
	view.z = -1.0f;
	right.x = 1.0f;
	up.y = 1.0f;
	angle.x = angle.y = angle.z = 0;
	//scale = 1.0f;

	textureCount = 0;
	ti = (TexInfo**)malloc(MAX_TEXCOUNT*sizeof(DWORD));
}

Model::~Model() {
    delete [] vertices;
    delete [] glcmds;
    delete [] lightnormals;

	for (int i=0;i<textureCount;i++)
		UnloadTexture(i);
}

void Model::UnloadTexture(int id) {
	free((void*)ti[id]);
	ti[id] = (TexInfo*) NULL;
}

void Model::Update() {
	//position += movesTo;
	//angle += rotation;
}

void Model::RotateX(float amount) {
	if (view.DotProduct(*(new Vector(0.0f,-1.0f,0.0f)))>8.5 && amount >0) return;

	view = Vector(view*cos(amount*PIdiv180) + up*sin(amount*PIdiv180));
	view.Normalise();
	
	//right.y = 0.0f;
	up = view.CrossProduct(right)*-1;
	right = view.CrossProduct(up);
	right.y = 0.0f;
	right.Normalise();
	angle.x += amount;
}

void Model::RotateY(float amount) {
	//rot.y += amount;
	view = Vector(view*cos(amount*PIdiv180) - right*sin(amount*PIdiv180));
	view.Normalise();
	
	right = view.CrossProduct(up);
	right.y = 0.0f;
	right.Normalise();
	up = view.CrossProduct(right)*-1;

	angle.y += amount;
}

void Model::RotateZ(float amount) {
	//rot.z += amount;

	right = Vector(right*cos(amount*PIdiv180) + up*sin(amount*PIdiv180));
	right.y = 0.0f;
	right.Normalise();

	up = view.CrossProduct(right)*-1;

	angle.z += amount;
}


void Model::MoveFwd(float amount) {
	//float y = pos.y;
	pos += view*amount;
	//pos.y = y;
}

void Model::MoveBack(float amount) {
	float y = pos.y;
	pos -= view*(amount);
	pos.y = y;
}

void Model::MoveLeft(float amount) {
	float y = pos.y;
	pos -= right*(amount);
	pos.y = y;
}

void Model::MoveRight(float amount) {
	float y = pos.y;
	pos += right*(amount);
	pos.y = y;
}

void Model::MoveUp(float amount) {
	pos += up*(amount);
}

void Model::MoveDown(float amount) {
	pos -= up*(amount);
}

void Model::SetModel(Vector pos, Vector view, Vector up, Vector right) {
	view.Normalise();
	right.Normalise();
	up.Normalise();
	this->pos = pos;
	this->view = view;
	this->right = right;
	this->up = up;
}