#pragma once

#include <cmath>
#include <iostream>
#include "global.h"

typedef float vec3_t[3];

class Vector {
public:
	float x, y, z;
	void Reset() { x = y = z = 0.0f; };
	void Normalise();
	float GetLength() { return sqrt(x*x + y*y + z*z); };
	float DotProduct(Vector &op);
	Vector CrossProduct(Vector &op);
	Vector() { x = y = z = 0.0f; };
	Vector(Vector &dup) { x = dup.x; y = dup.y; z = dup.z; };
	Vector(float x, float y, float z) { this->x = x; this->y = y; this->z = z; };
	Vector(vec3_t v) { x = v[0]; y = v[1]; z = v[2]; };
	Vector operator += (Vector &op) { x += op.x; y += op.y; z += op.z; return *this;};
	Vector operator -= (Vector &op) { x -= op.x; y -= op.y; z -= op.z; return *this;};
	Vector operator *= (float op) { x *= op; y *= op; z *= op; return *this;};
	Vector operator * (float op) { return Vector(x*op, y*op, z*op); };
	Vector operator + (Vector &op) { return Vector(x+op.x, y+op.y, z+op.z); };
	Vector operator - (Vector &op) { return Vector(x-op.x, y-op.y, z-op.z); };
};

